# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,invalid-name

import os

import pytest

from dump2polarion import properties, utils
from dump2polarion.exceptions import Dump2PolarionException
from tests import conf


# pylint: disable=too-many-public-methods
class TestProperties:
    def test_fill_testrun_present(self):
        fname = "complete_notransform.xml"
        fname = os.path.join(conf.DATA_PATH, fname)
        xml_root = utils.get_xml_root(fname)
        properties.xunit_fill_testrun_id(xml_root, "not_used")
        filled = utils.etree_to_string(xml_root)
        assert 'name="polarion-testrun-id" value="5_8_0_17"' in filled

    def test_fill_testrun_invalid_xml(self):
        xml_root = utils.get_xml_root_from_str("<invalid/>")
        with pytest.raises(Dump2PolarionException) as excinfo:
            properties.xunit_fill_testrun_id(xml_root, "5_8_0_17")
        assert "missing <testsuites>" in str(excinfo.value)

    def test_fill_testrun_no_properties(self):
        xml_root = utils.get_xml_root_from_str("<testsuites/>")
        with pytest.raises(Dump2PolarionException) as excinfo:
            properties.xunit_fill_testrun_id(xml_root, "5_8_0_17")
        assert "Failed to find <properties> in the XML file" in str(excinfo.value)

    def test_fill_testrun_missing(self):
        fname = "properties.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        properties.xunit_fill_testrun_id(xml_root, "5_8_0_17")
        filled = utils.etree_to_string(xml_root)
        assert 'name="polarion-testrun-id" value="5_8_0_17"' in filled

    @pytest.mark.parametrize("fname", ("testcases.xml", "complete_transform.xml"))
    def test_nofill_response(self, fname):
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = properties.fill_response_property(xml_root)
        assert name == "test"
        assert value == "test"

    @pytest.mark.parametrize("fname", ("testcases_noresponse.xml", "testcases_noresponse2.xml"))
    def test_fill_testcase_response(self, fname):
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = properties.fill_response_property(xml_root)
        filled = utils.etree_to_string(xml_root)
        assert name == "dump2polarion"
        assert value
        assert '<response-property name="dump2polarion" value=' in filled

    def test_fill_testsuites_response(self):
        fname = "complete_transform_noresponse.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = properties.fill_response_property(xml_root)
        filled = utils.etree_to_string(xml_root)
        assert name == "dump2polarion"
        assert value
        assert '<property name="polarion-response-dump2polarion" value=' in filled

    def test_fill_custom_testcase_response(self):
        fname = "testcases_noresponse.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = properties.fill_response_property(xml_root, "test", "test")
        filled = utils.etree_to_string(xml_root)
        assert name == "test"
        assert value == "test"
        assert '<response-property name="test" value="test"' in filled
        # make sure response properties are on top
        assert '<testcases project-id="RHCF3"><response-properties>' in filled

    def test_fill_custom_testsuites_response(self):
        fname = "complete_transform_noresponse.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = properties.fill_response_property(xml_root, "test", "test")
        filled = utils.etree_to_string(xml_root)
        assert name == "test"
        assert value == "test"
        assert '<property name="polarion-response-test" value="test"' in filled

    def test_fill_invalid_testsuites_response(self):
        xml_root = utils.get_xml_root_from_str("<invalid/>")
        with pytest.raises(Dump2PolarionException) as excinfo:
            properties.fill_response_property(xml_root, "test", "test")
        assert "XML file is not in expected format" in str(excinfo.value)

    def test_remove_testsuites_response_property(self):
        fname = "complete_transform.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert '<property name="polarion-response-test"' in filled
        properties.remove_response_property(xml_root)
        filled = utils.etree_to_string(xml_root)
        assert '<property name="polarion-response-test"' not in filled

    def test_remove_testcases_response_property(self):
        fname = "testcases.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert '<response-property name="test"' in filled
        properties.remove_response_property(xml_root)
        filled = utils.etree_to_string(xml_root)
        assert "<response-properties" not in filled

    def test_remove_testsuites_property(self):
        fname = "complete_transform.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert '<property name="polarion-testrun-id"' in filled
        properties.remove_property(xml_root, "testrun-id")
        filled = utils.etree_to_string(xml_root)
        assert '<property name="polarion-testrun-id"' not in filled

    def test_remove_testcases_property(self):
        fname = "testcases.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert '<property name="lookup-method"' in filled
        properties.remove_property(xml_root, "lookup-method")
        filled = utils.etree_to_string(xml_root)
        assert '<property name="lookup-method"' not in filled

    def test_add_testsuites_lookup_property(self):
        fname = "complete_noprop.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert "polarion-lookup-method" not in filled
        properties.set_lookup_method(xml_root, "name")
        filled = utils.etree_to_string(xml_root)
        assert 'property name="polarion-lookup-method" value="name"' in filled

    def test_add_testcases_lookup_property(self):
        fname = "testcases_noprop.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert "lookup-method" not in filled
        properties.set_lookup_method(xml_root, "name")
        filled = utils.etree_to_string(xml_root)
        assert 'property name="lookup-method" value="name"' in filled

    def test_add_testsuites_dry_run(self):
        fname = "complete_noprop.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert "polarion-dry-run" not in filled
        properties.set_dry_run(xml_root, True)
        filled = utils.etree_to_string(xml_root)
        assert 'property name="polarion-dry-run" value="true"' in filled

    def test_add_testcases_dry_run(self):
        fname = "testcases_noprop.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        filled = utils.etree_to_string(xml_root)
        assert "dry-run" not in filled
        properties.set_dry_run(xml_root, True)
        filled = utils.etree_to_string(xml_root)
        assert 'property name="dry-run" value="true"' in filled



"""
Allowed values for verdicts.
"""


class Verdicts:
    """Valid verdicts."""

    PASS = ("passed", "pass")
    FAIL = ("failed", "fail")
    SKIP = ("skipped", "skip", "blocked")
    WAIT = ("null", "wait", "waiting")



# pylint: disable=missing-docstring,no-self-use

import copy
import io
import logging
import os
from collections import OrderedDict

import pytest

from tests import conf

GENERIC_CONF = {
    "xunit_target": "https://polarion.example.com/import/xunit",
    "testcase_taget": "https://polarion.example.com/import/testcase",
    "requirement_target": "https://polarion.example.com/import/requirement",
    "xunit_queue": "https://polarion.example.com/import/xunit-queue",
    "testcase_queue": "https://polarion.example.com/import/testcase-queue",
    "requirement_queue": "https://polarion.example.com/import/requirement-queue",
    "repo_address": "https://foo/bar/baz",
    "cfme_run_id": "id123",
}


VALID_CASELEVELS = {
    "valid_values": {"caselevel": ["component", "integration", "system", "acceptance"]}
}

RHCF3_XUNIT_PROPS = OrderedDict(
    (
        ("polarion-dry-run", False),
        ("polarion-testrun-status-id", "inprogress"),
        ("polarion-response-test", "test"),
    )
)

POLARION_DEFAULT_FIELDS = OrderedDict(
    (
        ("assignee", ""),
        ("casecomponent", ""),
        ("initialEstimate", "1m"),
        ("caseimportance", "high"),
        ("caselevel", "component"),
        ("caseposneg", "positive"),
        ("caseautomation", "automated"),
        ("testtype", "functional"),
        ("subtype1", "-"),
        ("subtype2", "-"),
        ("tags", ""),
        ("setup", ""),
        ("teardown", ""),
        ("description", ""),
        ("linkedWorkItems", ""),
        ("testSteps", ""),
        ("expectedResults", ""),
        ("title", ""),
        ("work_item_id", ""),
    )
)

REQUIREMENTS_DEFAULT_FIELDS = OrderedDict((("reqtype", "system"),))

RHCF3_TESTCASE_PROPS = OrderedDict((("lookup-method", "name"),))

RHCF3_CONF = GENERIC_CONF.copy()
RHCF3_CONF["polarion-project-id"] = "RHCF3"
RHCF3_CONF["xunit_import_properties"] = RHCF3_XUNIT_PROPS
RHCF3_CONF["testcase_import_properties"] = RHCF3_TESTCASE_PROPS
RHCF3_CONF["default_fields"] = POLARION_DEFAULT_FIELDS
RHCF3_CONF["requirements_default_fields"] = REQUIREMENTS_DEFAULT_FIELDS
RHCF3_CONF["docstrings"] = VALID_CASELEVELS

CMP_CONF = GENERIC_CONF.copy()
CMP_CONF["polarion-project-id"] = "CMP"
CMP_CONF["xunit_import_properties"] = RHCF3_XUNIT_PROPS.copy()
CMP_CONF["default_fields"] = POLARION_DEFAULT_FIELDS.copy()
CMP_CONF["docstrings"] = VALID_CASELEVELS.copy()


@pytest.fixture(scope="session")
def config_prop():
    return copy.deepcopy(RHCF3_CONF)


@pytest.fixture(scope="session")
def config_prop_cmp():
    return copy.deepcopy(CMP_CONF)


@pytest.fixture(scope="function")
def config_e2e():
    conf_file = os.path.join(conf.DATA_PATH, "polarion_tools.yaml")
    return str(conf_file)


class SimpleFormatter:
    def format(self, record):
        message = record.getMessage()
        if isinstance(message, bytes):
            message = message.decode("utf-8")
        return message


@pytest.yield_fixture
def captured_log():
    buff = io.StringIO()
    handler = logging.StreamHandler(buff)
    handler.setFormatter(SimpleFormatter())

    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    logger.addHandler(handler)

    yield buff

    logger.handlers.remove(handler)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import os

import pytest

from dump2polarion import parselogs
from dump2polarion.exceptions import Dump2PolarionException
from tests import conf


class TestParselog:
    def test_xunit_name(self):
        log_file = os.path.join(conf.DATA_PATH, "xunit.log")
        parsed_log = parselogs.parse(log_file)
        assert parsed_log.log_type == "xunit"
        assert len(parsed_log.duplicate_items) == 10
        assert len(parsed_log.existing_items) == 291
        assert len(parsed_log.new_items) == 26
        eitem = parsed_log.existing_items[10]
        assert eitem.id == "RHCF3-47696"
        assert eitem.name == "test_collections_actions[virtualcenter-users]"
        assert not eitem.custom_id

    def test_xunit_custom_id(self):
        log_file = os.path.join(conf.DATA_PATH, "xunit_vmaas.log")
        parsed_log = parselogs.parse(log_file)
        assert parsed_log.log_type == "xunit"
        assert not parsed_log.duplicate_items
        assert not parsed_log.new_items
        assert len(parsed_log.existing_items) == 168
        eitem = parsed_log.existing_items[10]
        assert eitem.id == "INSI-1547"
        assert eitem.name == "TestUpdateInOtherRepo.test_post_single"
        assert eitem.custom_id == "4267c8b00cf1f4d48c9565aa166c318e"

    def test_xunit_invalid(self):
        with pytest.raises(Dump2PolarionException):
            parselogs.XUnitParser([], "empty").parse()

    def test_testcase_custom_id(self):
        log_file = os.path.join(conf.DATA_PATH, "testcase.log")
        parsed_log = parselogs.parse(log_file)
        assert parsed_log.log_type == "testcase"
        assert not parsed_log.new_items
        assert len(parsed_log.duplicate_items) == 1
        assert len(parsed_log.existing_items) == 82
        eitem = parsed_log.existing_items[10]
        assert eitem.id == "INSI-1498"
        assert eitem.name == "TestCVEsCorrect.test_post_single_smoke"
        assert eitem.custom_id == "0422b43ddc9ccc44f2cc5358ac25ba58"

    def test_testcase_invalid(self):
        with pytest.raises(Dump2PolarionException):
            parselogs.TestcasesParser([], "empty").parse()

    def test_requirement_name(self):
        log_file = os.path.join(conf.DATA_PATH, "requirements.log")
        parsed_log = parselogs.parse(log_file)
        assert parsed_log.log_type == "requirement"
        assert len(parsed_log.duplicate_items) == 1
        assert len(parsed_log.new_items) == 49
        assert len(parsed_log.existing_items) == 1
        eitem = parsed_log.new_items[10]
        assert eitem.id == "INSI-1683"
        assert eitem.name == "configuration"
        assert not eitem.custom_id

    def test_requirement_invalid(self):
        with pytest.raises(Dump2PolarionException):
            parselogs.RequirementsParser([], "empty").parse()

    def test_log_invalid(self, tmpdir):
        invalid_log = os.path.join(str(tmpdir), "invalid.log")
        with open(invalid_log, "w") as output_file:
            output_file.write("foo\n")
        with pytest.raises(Dump2PolarionException):
            parselogs.parse(invalid_log)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import datetime
import os
import shutil

import pytest

from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.exporters.xunit_exporter import XunitExport
from dump2polarion.results import dbtools
from tests import conf


@pytest.fixture(scope="module")
def records_db():
    db_file = os.path.join(conf.DATA_PATH, "workitems_ids.sqlite3")
    return dbtools.import_sqlite(db_file)


class TestDB:
    def test_testrun_id_exported(self):
        db_file = os.path.join(conf.DATA_PATH, "workitems_ids.sqlite3")
        conn = dbtools._open_sqlite(db_file)
        testrun_id = dbtools._get_testrun_from_sqlite(conn)
        assert testrun_id == "5_8_0_17"

    def test_testrun_invalid_db(self):
        db_file = os.path.join(conf.DATA_PATH, "ostriz.json")
        conn = dbtools._open_sqlite(db_file)
        testrun_id = dbtools._get_testrun_from_sqlite(conn)
        assert testrun_id is None

    def test_import_orig_data(self, records_db):
        assert hasattr(records_db, "results")
        assert len(records_db.results) == 15
        assert "id" in records_db.results[0]
        assert hasattr(records_db, "testrun")
        assert records_db.testrun == "5_8_0_17"

    def test_import_time(self):
        db_file = os.path.join(conf.DATA_PATH, "workitems_ids.sqlite3")
        older_than = datetime.datetime(2017, 6, 15)
        records = dbtools.import_sqlite(db_file, older_than=older_than)
        assert hasattr(records, "results")
        assert len(records.results) == 14

    def test_open_nonexistent(self):
        db_file = "nonexistent"
        with pytest.raises(Dump2PolarionException) as excinfo:
            dbtools._open_sqlite(db_file)
        assert "No such file or directory" in str(excinfo.value)

    def test_exported_older_than(self, tmpdir):
        orig_db_file = os.path.join(conf.DATA_PATH, "workitems_ids.sqlite3")
        older_than = datetime.datetime(2017, 6, 15)
        db_file = os.path.join(str(tmpdir), "workitems_copy.sqlite3")
        shutil.copy(orig_db_file, db_file)
        dbtools.mark_exported_sqlite(db_file, older_than=older_than)
        conn = dbtools._open_sqlite(db_file)
        cur = conn.cursor()
        select = "SELECT count(*) FROM testcases WHERE exported == 'yes'"
        cur.execute(select)
        num = cur.fetchone()
        conn.close()
        assert num[0] == 12

    def test_exported_all(self, tmpdir):
        orig_db_file = os.path.join(conf.DATA_PATH, "workitems_ids.sqlite3")
        db_file = os.path.join(str(tmpdir), "workitems_copy.sqlite3")
        shutil.copy(orig_db_file, db_file)
        dbtools.mark_exported_sqlite(db_file)
        conn = dbtools._open_sqlite(db_file)
        cur = conn.cursor()
        select = "SELECT count(*) FROM testcases WHERE exported == 'yes'"
        cur.execute(select)
        num = cur.fetchone()
        conn.close()
        assert num[0] == 13

    def test_e2e_ids_notransform(self, config_prop, records_db):
        exporter = XunitExport("5_8_0_17", records_db, config_prop, transform_func=lambda arg: arg)
        complete = exporter.export()
        fname = "complete_notransform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_transform(self, config_prop, records_db):
        exporter = XunitExport("5_8_0_17", records_db, config_prop)
        complete = exporter.export()
        fname = "complete_transform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed



"""
Access work items data in the Polarion SVN repository.
"""

import logging
import os
from collections import defaultdict

from lxml import etree

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


class InvalidObject:
    """Item not present."""


class WorkItemCache:
    """Cache of Polarion workitems."""

    def __init__(self, repo_dir):
        self.repo_dir = repo_dir
        self.test_case_dir = os.path.join(self.repo_dir, "tracker", "workitems")
        self._cache = defaultdict(dict)

    @staticmethod
    def get_path(num):
        """Gets a path from the workitem number.

        For example: 31942 will return 30000-39999/31000-31999/31900-31999
        """
        num = int(num)
        dig_len = len(str(num))
        paths = []
        for i in range(dig_len - 2):
            divisor = 10 ** (dig_len - i - 1)
            paths.append(
                "{}-{}".format((num // divisor) * divisor, (((num // divisor) + 1) * divisor) - 1)
            )
        return "/".join(paths)

    def get_tree(self, work_item_id):
        """Gets XML tree of the workitem."""
        try:
            __, tcid = work_item_id.split("-")
        except ValueError:
            logger.warning("Couldn't load workitem %s, bad format", work_item_id)
            self._cache[work_item_id] = InvalidObject()
            return None

        path = os.path.join(self.test_case_dir, self.get_path(tcid), work_item_id, "workitem.xml")
        try:
            tree = etree.parse(path)
        # pylint: disable=broad-except
        except Exception:
            logger.warning("Couldn't load workitem %s", work_item_id)
            self._cache[work_item_id] = InvalidObject()
            return None
        return tree

    @staticmethod
    def _get_steps(item):
        steps = []
        expected_results = []

        steps_list = item.xpath('.//item[@id = "steps"]')
        if not steps_list:
            return steps, expected_results

        steps_list = steps_list[0]
        steps_items = steps_list.xpath(".//item[@text-type]")
        for index, rec in enumerate(steps_items):
            if index % 2 == 0:
                steps.append(rec.text)
            else:
                expected_results.append(rec.text)
        return steps, expected_results

    @staticmethod
    def _get_linked_items(item):
        linked = []
        linked_items = item.xpath("./list/struct")
        for struct in linked_items:
            role_found = False
            struct_item = None

            for litem in struct:
                attrib = litem.attrib["id"]
                if attrib == "role":
                    if litem.text == "verifies":
                        role_found = True
                    else:
                        break
                elif attrib == "workItem":
                    struct_item = litem.text

            if role_found and struct_item:
                linked.append(struct_item)

        return linked

    def __getitem__(self, work_item_id):
        if work_item_id in self._cache:
            return self._cache[work_item_id]
        if isinstance(self._cache[work_item_id], InvalidObject):
            return None

        tree = self.get_tree(work_item_id)
        if not tree:
            return None

        for item in tree.xpath("/work-item/field"):
            attrib = item.attrib["id"]
            if attrib == "testSteps":
                steps, results = self._get_steps(item)
                self._cache[work_item_id]["testSteps"] = steps
                self._cache[work_item_id]["expectedResults"] = results
            elif attrib == "linkedWorkItems":
                self._cache[work_item_id]["linkedWorkItems"] = self._get_linked_items(item)
            else:
                self._cache[work_item_id][item.attrib["id"]] = item.text

        self._cache[work_item_id]["work_item_id"] = work_item_id
        if "assignee" not in self._cache[work_item_id]:
            self._cache[work_item_id]["assignee"] = ""
        if "title" not in self._cache[work_item_id]:
            logger.debug("Workitem %s has no title", work_item_id)

        return self._cache[work_item_id]

    def get_all_items(self):
        """Walks the repo and returns work items."""
        for item in os.walk(self.test_case_dir):
            if "workitem.xml" not in item[2]:
                continue
            case_id = os.path.split(item[0])[-1]
            if not (case_id and "*" not in case_id):
                continue
            item_cache = self[case_id]
            if not item_cache:
                continue
            if not item_cache.get("title"):
                continue
            yield item_cache



"""
Helper functions for handling data in pytest junit format.
"""

import os

from lxml import etree

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.exporters import xunit_exporter

_PARAMETER_PREFIX = "polarion-parameter-"


def _get_xml_root(junit_file):
    try:
        tree = etree.parse(os.path.expanduser(junit_file))
    except Exception as err:
        raise Dump2PolarionException("Failed to parse XML file '{}': {}".format(junit_file, err))

    return tree.getroot()


def _parse_testcase_record(testcase_record):
    """Parses testcase record and returns it's info."""
    verdict = None
    verdict_found = False
    comment = ""
    properties = {}
    for element in testcase_record:
        if not verdict_found:
            if element.tag == "error":
                verdict = "failed"
                comment = element.get("message")
                # continue to see if there's more telling verdict for this record
            elif element.tag == "failure":
                verdict = "failed"
                comment = element.get("message")
                verdict_found = True
            elif element.tag == "skipped":
                verdict = "skipped"
                comment = element.get("message")
                verdict_found = True
        if element.tag == "properties":
            for prop in element:
                properties[prop.get("name")] = prop.get("value")
    if not verdict:
        verdict = "passed"

    return verdict, comment, properties


def _extract_parameters_from_properties(properties):
    """Extracts parameters from properties."""
    new_properties = {}
    parameters = {}
    for key, value in properties.items():
        if key.startswith(_PARAMETER_PREFIX):
            parameters[key.replace(_PARAMETER_PREFIX, "")] = value
        else:
            new_properties[key] = value

    return new_properties, parameters


# pylint: disable=unused-argument,too-many-locals
def import_junit(junit_file, **kwargs):
    """Reads the content of the junit-results file produced by pytest and returns imported data."""
    xml_root = _get_xml_root(junit_file)

    testcases = xml_root.xpath(".//testcase")
    if testcases is None:
        testcases = ()

    results = []
    for test_data in testcases:
        verdict, comment, properties = _parse_testcase_record(test_data)
        properties, parameters = _extract_parameters_from_properties(properties)

        testcase_id = properties.get("polarion-testcase-id")
        title = test_data.get("name")
        classname = test_data.get("classname")
        time = test_data.get("time", 0)
        filepath = test_data.get("file")

        data = {
            "id": testcase_id,
            "title": title,
            "classname": classname,
            "verdict": verdict,
            "comment": comment,
            "time": time,
            "file": filepath,
        }
        for key, value in properties.items():
            data[key] = value
        if parameters:
            data["params"] = utils.sorted_dict(parameters)

        results.append(utils.sorted_dict(data))

    return xunit_exporter.ImportedData(results=results, testrun=None)



"""
Helper functions for transforming results.
"""

import hashlib
import logging
import os
import re

from docutils.core import publish_parts

from dump2polarion.exporters.verdicts import Verdicts

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)

TEST_PARAM_RE = re.compile(r"\[.*\]")


def only_passed_and_wait(result):
    """Returns PASS and WAIT results only, skips everything else."""
    verdict = result.get("verdict", "").strip().lower()
    if verdict in Verdicts.PASS + Verdicts.WAIT:
        return result
    return None


def insert_source_info(result):
    """Adds info about source of test result if available."""
    comment = result.get("comment")
    # don't change comment if it already exists
    if comment:
        return

    source = result.get("source")
    job_name = result.get("job_name")
    run = result.get("run")
    source_list = [source, job_name, run]
    if not all(source_list):
        return

    source_note = "/".join(source_list)
    source_note = "Source: {}".format(source_note)
    result["comment"] = source_note


def setup_parametrization(result, parametrize):
    """Modifies result's data according to the parametrization settings."""
    if parametrize:
        # remove parameters from title
        title = result.get("title")
        if title:
            result["title"] = TEST_PARAM_RE.sub("", title)
        # remove parameters also from id if it's identical to title
        if title and result.get("id") == title:
            result["id"] = result["title"]
    else:
        # don't parametrize if not specifically configured
        if "params" in result:
            del result["params"]


def include_class_in_title(result):
    """Makes sure that test class is included in "title".

    Applies only to titles derived from test function names, e.g.
    "test_power_parent_service" -> "TestServiceRESTAPI.test_power_parent_service"

    >>> result = {"title": "test_foo", "id": "test_foo", "classname": "foo.bar.baz.TestFoo",
    ...    "file": "foo/bar/baz.py"}
    >>> include_class_in_title(result)
    >>> str(result.get("title"))
    'TestFoo.test_foo'
    >>> str(result.get("id"))
    'TestFoo.test_foo'
    >>> result.get("classname")
    >>> result = {"title": "some title", "id": "test_foo", "classname": "foo.bar.baz.TestFoo",
    ...    "file": "foo/bar/baz.py"}
    >>> include_class_in_title(result)
    >>> str(result.get("title"))
    'some title'
    >>> str(result.get("id"))
    'test_foo'
    """
    classname = result.get("classname", "")
    if not classname:
        return

    filepath = result.get("file", "")
    title = result.get("title")
    if title and title.startswith("test_") and "/" in filepath and "." in classname:
        fname = filepath.split("/")[-1].replace(".py", "")
        last_classname = classname.split(".")[-1]
        # last part of classname is not file name
        if fname != last_classname and last_classname not in title:
            result["title"] = "{}.{}".format(last_classname, title)
        # update also the id if it is identical to original title
        if result.get("id") == title:
            result["id"] = result["title"]

    # we don't need to pass classnames?
    del result["classname"]


def gen_unique_id(string):
    """Generates unique id out of a string.

    >>> gen_unique_id("vmaas_TestClass.test_name")
    '5acc5dc795a620c6b4491b681e5da39c'
    """
    return hashlib.sha1(string.encode("utf-8")).hexdigest()[:32]


def get_testcase_id(testcase, append_str):
    """Returns new test case ID.

    >>> get_testcase_id({"title": "TestClass.test_name"}, "vmaas_")
    '5acc5dc795a620c6b4491b681e5da39c'
    >>> get_testcase_id({"title": "TestClass.test_name", "id": "TestClass.test_name"}, "vmaas_")
    '5acc5dc795a620c6b4491b681e5da39c'
    >>> get_testcase_id({"title": "TestClass.test_name", "id": "test_name"}, "vmaas_")
    '5acc5dc795a620c6b4491b681e5da39c'
    >>> get_testcase_id({"title": "some title", "id": "TestClass.test_name"}, "vmaas_")
    '2ea7695b73763331f8a0c4aec75362b8'
    >>> str(get_testcase_id({"title": "some title", "id": "some_id"}, "vmaas_"))
    'some_id'
    """
    testcase_title = testcase.get("title")
    testcase_id = testcase.get("id")
    if not testcase_id or testcase_id.lower().startswith("test"):
        testcase_id = gen_unique_id("{}{}".format(append_str, testcase_title))
    return testcase_id


def parse_rst_description(testcase):
    """Creates an HTML version of the RST formatted description."""
    description = testcase.get("description")

    if not description:
        return

    try:
        with open(os.devnull, "w") as devnull:
            testcase["description"] = publish_parts(
                description,
                writer_name="html",
                settings_overrides={"report_level": 2, "halt_level": 2, "warning_stream": devnull},
            )["html_body"]
    # pylint: disable=broad-except
    except Exception as exp:
        testcase_id = testcase.get("nodeid") or testcase.get("id") or testcase.get("title")
        logger.error("%s: description: %s", str(exp), testcase_id)


def preformat_plain_description(testcase):
    """Creates a preformatted HTML version of the description."""
    description = testcase.get("description")

    if not description:
        return

    # naive approach to removing indent from pytest docstrings
    nodeid = testcase.get("nodeid") or ""
    indent = None
    if "::Test" in nodeid:
        indent = 8 * " "
    elif "::test_" in nodeid:
        indent = 4 * " "

    if indent:
        orig_lines = description.split("\n")
        new_lines = []
        for line in orig_lines:
            if line.startswith(indent):
                line = line.replace(indent, "", 1)
            new_lines.append(line)
        description = "\n".join(new_lines)

    testcase["description"] = "<pre>\n{}\n</pre>".format(description)


def add_unique_runid(testcase, run_id=None):
    """Adds run id to the test description.

    The `run_id` runs makes the descriptions unique between imports and force Polarion
    to update every testcase every time.
    """
    testcase["description"] = '{visible}<br id="{invisible}"/>'.format(
        visible=testcase.get("description") or "empty-description-placeholder",
        invisible=run_id or id(add_unique_runid),
    )


def add_automation_link(testcase):
    """Appends link to automation script to the test description."""
    automation_link = (
        '<a href="{}">Test Source</a>'.format(testcase["automation_script"])
        if testcase.get("automation_script")
        else ""
    )
    testcase["description"] = "{}<br/>{}".format(testcase.get("description") or "", automation_link)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import copy
import os
from collections import OrderedDict

import pytest

from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporters.testcases_exporter import TestcaseExport
from tests import conf

REQ_DATA = [
    OrderedDict(
        (
            ("id", "ITEM01"),
            ("title", "test_manual"),
            ("description", "Manual tests with many supported fields."),
            ("approver-ids", "mkourim:approved"),
            ("assignee-id", "mkourim"),
            ("due-date", "2018-09-30"),
            ("initial-estimate", "1/4h"),
            ("caseautomation", "manualonly"),
            ("caseimportance", "high"),
            ("caselevel", 2),
            ("caseposneg", "positive"),
            ("testtype", "functional"),
            ("subtype1", "-"),
            ("subtype2", "-"),
            ("upstream", "yes"),
            ("tags", "tag1, tag2"),
            ("setup", "Do this first"),
            ("teardown", "Clean up"),
            ("automation_script", "file#L83"),
            ("testSteps", ["step1", "step2"]),
            ("expectedResults", ["result1", "result2"]),
            ("linked-items", "ITEM01"),
            ("unknown", "non-included"),
        )
    ),
    OrderedDict(
        (
            ("title", "test_minimal_param"),
            ("params", ["param1", "param2"]),
            ("linked-items-lookup-method", "name"),
            ("linked-items", [{"id": "ITEM01", "role": "derived_from"}]),
            ("automation_script", "https://github.com/somerepo/blob/somebranch/file#L83"),
        )
    ),
]


@pytest.fixture(scope="module")
def config_cloudtp(config_prop):
    cloudtp = copy.deepcopy(config_prop)
    cloudtp["polarion-project-id"] = "CLOUDTP"
    cloudtp["testcase_import_properties"] = {"prop1": "val1", "prop2": "val2"}
    return cloudtp


@pytest.fixture(scope="module")
def config_with_fields(config_prop):
    config_fields = copy.deepcopy(config_prop)
    config_fields["default_fields"] = {"caseimportance": "medium", "startsin": "5.10"}
    config_fields["custom_fields"] = ["caseimportance", "startsin", "testtype"]
    return config_fields


class TestTestcase:
    def test_export_cloudtp(self, config_cloudtp):
        testcase_exp = TestcaseExport(REQ_DATA, config_cloudtp)
        complete = testcase_exp.export()
        fname = "testcase_complete_cloudtp.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_export_cfme(self, config_prop):
        testcase_exp = TestcaseExport(REQ_DATA, config_prop)
        complete = testcase_exp.export()
        fname = "testcase_complete_cfme.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_export_fields_cfme(self, config_with_fields):
        testcase_exp = TestcaseExport(REQ_DATA, config_with_fields)
        complete = testcase_exp.export()
        fname = "testcase_fields_cfme.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_export_cfme_params(self, config_prop):
        new_config = copy.deepcopy(config_prop)
        new_config["cfme_parametrize"] = True
        testcase_exp = TestcaseExport(REQ_DATA, new_config)
        complete = testcase_exp.export()
        fname = "testcase_complete_cfme_param.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_invalid_lookup(self, config_cloudtp):
        new_config = copy.deepcopy(config_cloudtp)
        new_config["testcase_import_properties"] = {"lookup-method": "inv"}
        testcase_exp = TestcaseExport(REQ_DATA, new_config)
        with pytest.raises(Dump2PolarionException) as excinfo:
            testcase_exp.export()
        assert "Invalid value 'inv' for the 'lookup-method' property" in str(excinfo.value)

    def test_no_requirements(self, config_cloudtp):
        testcase_exp = TestcaseExport([], config_cloudtp)
        with pytest.raises(NothingToDoException) as excinfo:
            testcase_exp.export()
        assert "Nothing to export" in str(excinfo.value)



"""
Dump testcases results from a CSV input file to SQLite.
"""

import argparse
import datetime
import logging
import os
import sqlite3

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.results import csvtools

REQUIRED_KEYS = (
    "verdict",
    "last_status",
    "exported",
    "time",
    "comment",
    "stdout",
    "stderr",
    "user1",
)

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


def get_args(args=None):
    """Get command line arguments."""
    parser = argparse.ArgumentParser(description="csv2sqlite")
    parser.add_argument("-i", "--input_file", required=True, help="Path to CSV records file")
    parser.add_argument("-o", "--output_file", required=True, help="Path to sqlite output file")
    parser.add_argument("--log-level", help="Set logging to specified level")
    return parser.parse_args(args)


def dump2sqlite(records, output_file):
    """Dumps tests results to database."""
    results_keys = list(records.results[0].keys())
    pad_data = []

    for key in REQUIRED_KEYS:
        if key not in results_keys:
            results_keys.append(key)
            pad_data.append("")

    conn = sqlite3.connect(os.path.expanduser(output_file), detect_types=sqlite3.PARSE_DECLTYPES)

    # in each row there needs to be data for every column
    # last column is current time
    pad_data.append(datetime.datetime.utcnow())

    to_db = [list(row.values()) + pad_data for row in records.results]

    cur = conn.cursor()
    cur.execute(
        "CREATE TABLE testcases ({},sqltime TIMESTAMP)".format(
            ",".join("{} TEXT".format(key) for key in results_keys)
        )
    )
    cur.executemany(
        "INSERT INTO testcases VALUES ({},?)".format(",".join(["?"] * len(results_keys))), to_db
    )

    if records.testrun:
        cur.execute("CREATE TABLE testrun (testrun TEXT)")
        cur.execute("INSERT INTO testrun VALUES (?)", (records.testrun,))

    conn.commit()
    conn.close()

    logger.info("Data written to '%s'", output_file)


def main(args=None):
    """Main function for cli."""
    args = get_args(args)

    utils.init_log(args.log_level)

    if ".csv" not in args.input_file.lower():
        logger.warning("Make sure the input file '%s' is in CSV format", args.input_file)

    try:
        records = csvtools.get_imported_data(args.input_file)
    except (OSError, Dump2PolarionException) as err:
        logger.fatal(err)
        return 1

    # check if all columns required by `pytest_polarion_cfme` are there
    required_columns = {"id": "ID", "title": "Title"}
    missing_columns = [required_columns[k] for k in required_columns if k not in records.results[0]]
    if missing_columns:
        logger.fatal(
            "The input file '%s' is missing following columns: %s",
            args.input_file,
            ", ".join(missing_columns),
        )
        return 1

    try:
        dump2sqlite(records, args.output_file)
    # pylint: disable=broad-except
    except Exception as err:
        logger.exception(err)
        return 1

    return 0



# pylint: disable=missing-docstring,no-self-use,protected-access

import os

from mock import patch

from dump2polarion import submit, utils
from tests import conf


# pylint: disable=unused-argument
class DummySession:
    def __init__(self, method):
        self._method = method

    def get(self, *args, **kwargs):
        return self._method()

    def post(self, *args, **kwargs):
        return self._method()


class DummyResponse:
    def __init__(self, response=None):
        self.status_code = 200
        self.response = response or {}
        self.url = "foo"

    def __len__(self):
        return 1

    def json(self):
        return self.response

    @property
    def content(self):
        return self.response


class TestSubmitAndVerify:
    def test_missing_input(self, config_prop, captured_log):
        submit.submit_and_verify("", config=config_prop, no_verify=True)
        assert "no data supplied" in captured_log.getvalue()

    def test_missing_credentials(self, config_prop, captured_log):
        submit.submit_and_verify("<testcases/>", config=config_prop, no_verify=True)
        assert "missing credentials" in captured_log.getvalue()

    def test_missing_target(self, config_prop, captured_log):
        submit.submit_and_verify("<foo/>", config=config_prop, no_verify=True)
        assert "submit target not found" in captured_log.getvalue()

    def test_missing_testrun_id(self, config_prop, captured_log):
        submit.submit_and_verify(
            "<testsuites><properties></properties></testsuites>",
            config=config_prop,
            user="john",
            password="123",
            session="foo",
            no_verify=True,
        )
        assert "missing testrun id" in captured_log.getvalue()

    def test_fill_testrun_submit(self, config_prop):
        fname = "properties.xml"
        submit.submit_and_verify(
            xml_file=os.path.join(conf.DATA_PATH, fname),
            testrun_id="5_8_0_17",
            config=config_prop,
            user="john",
            password="123",
            session=DummySession(DummyResponse),
            no_verify=True,
        )

    def test_get_testcases_taget(self, config_prop):
        xml_root = utils.get_xml_root_from_str("<testcases/>")
        submit_config = submit.SubmitConfig(xml_root, config_prop, user="john", password="123")
        assert "testcase" in submit_config.submit_target

    def test_get_testsuites_taget(self, config_prop):
        xml_root = utils.get_xml_root_from_str("<testsuites/>")
        submit_config = submit.SubmitConfig(xml_root, config_prop, user="john", password="123")
        assert "xunit" in submit_config.submit_target

    def test_get_requirements_taget(self, config_prop):
        xml_root = utils.get_xml_root_from_str("<requirements/>")
        submit_config = submit.SubmitConfig(xml_root, config_prop, user="john", password="123")
        assert "requirement" in submit_config.submit_target

    def test_file_testsuites_failure(self, config_prop, captured_log):
        class Response:
            def __init__(self):
                self.status_code = 404
                self.url = "foo"

            def __len__(self):
                return 0

        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        response = submit.submit_and_verify(
            xml_file=input_file,
            config=config_prop,
            user="john",
            password="123",
            session=DummySession(Response),
            no_verify=True,
        )
        assert not response
        assert "HTTP status 404: failed to submit" in captured_log.getvalue()

    def test_file_testsuites_none(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        response = submit.submit_and_verify(
            xml_file=input_file,
            config=config_prop,
            user="john",
            password="123",
            session=DummySession(lambda: None),
            no_verify=True,
        )
        assert not response
        assert "Failed to submit" in captured_log.getvalue()

    def test_file_testsuites_exception(self, config_prop, captured_log):
        def _raise():
            raise Exception("request failed")

        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        response = submit.submit_and_verify(
            xml_file=input_file,
            config=config_prop,
            user="john",
            password="123",
            session=DummySession(_raise),
            no_verify=True,
        )
        assert not response
        logged_data = captured_log.getvalue()
        assert "request failed" in logged_data
        assert "Failed to submit" in logged_data

    def test_file_testsuites_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        response = submit.submit_and_verify(
            xml_file=input_file,
            config=config_prop,
            user="john",
            password="123",
            session=DummySession(
                lambda: DummyResponse({"files": {"results.xml": {"job-ids": [1, 2]}}})
            ),
            no_verify=True,
        )
        assert response
        logged_data = captured_log.getvalue()
        assert "Results received" in logged_data
        assert "Job IDs" in logged_data

    def test_file_testcases_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "testcases.xml")
        response = submit.submit_and_verify(
            xml_file=input_file,
            config=config_prop,
            user="john",
            password="123",
            session=DummySession(
                lambda: DummyResponse({"files": {"results.xml": {"job-ids": [1, 2]}}})
            ),
            no_verify=True,
        )
        assert response
        logged_data = captured_log.getvalue()
        assert "Results received" in logged_data
        assert "Job ID" in logged_data

    def test_no_verify(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        response = submit.submit_and_verify(
            xml_file=input_file,
            config=config_prop,
            user="john",
            password="123",
            session=DummySession(
                lambda: DummyResponse({"files": {"results.xml": {"job-ids": [1, 2]}}})
            ),
            no_verify=True,
        )
        assert response
        assert "Results received" in captured_log.getvalue()

    def test_verify_failed(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        with patch("dump2polarion.verify.QueueSearch") as mock:
            instance = mock.return_value
            instance.verify_submit.return_value = False
            response = submit.submit_and_verify(
                xml_file=input_file,
                config=config_prop,
                user="john",
                password="123",
                session=DummySession(
                    lambda: DummyResponse({"files": {"results.xml": {"job-ids": [1, 2]}}})
                ),
            )
        assert not response
        assert "Results received" in captured_log.getvalue()

    def test_verify_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        with patch("dump2polarion.verify.QueueSearch") as mock:
            instance = mock.return_value
            instance.verify_submit.return_value = True
            response = submit.submit_and_verify(
                xml_file=input_file,
                config=config_prop,
                user="john",
                password="123",
                session=DummySession(
                    lambda: DummyResponse({"files": {"results.xml": {"job-ids": [1, 2]}}})
                ),
            )
        assert response
        assert "Results received" in captured_log.getvalue()



# pylint: disable-all

import warnings

from dump2polarion.exporters.testcases_exporter import *  # noqa

warnings.warn(
    "The testcases_exporter module was moved to 'dump2polarion.exporters.testcases_exporter'",
    DeprecationWarning,
)



"""
Dump testcases results to XUnit file for submitting to the Polarion XUnit Importer.

Example of input tests_records:
tests_records = ImportedData(
    results=[
        {
            "title": "test_post_single[vim-common-2:7.4.160-1.el7.x86_64.rpm]",
            "classname": "vmaas.tests.test_packages.TestPackagesQuery",
            "verdict": "passed",
            "time": "0.00890207290649414",
            "file": "vmaas/tests/test_packages.py",
            "params": {"pkg": "some_package"},
        }
    ],
    testrun=None,
)
"""

import datetime
import logging
from collections import namedtuple

from lxml import etree

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporters import transform_projects
from dump2polarion.exporters.verdicts import Verdicts

ImportedData = namedtuple("ImportedData", "results testrun")

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


class XunitExport:
    """Exports testcases results into Polarion XUnit."""

    def __init__(self, testrun_id, tests_records, config, transform_func=None):
        self.testrun_id = testrun_id
        self.tests_records = tests_records
        self.config = config
        self._lookup_prop = ""
        self._transform_func = transform_func or transform_projects.get_xunit_transform(config)

    def _top_element(self):
        """Returns top XML element."""
        top = etree.Element("testsuites")
        comment = etree.Comment("Generated for testrun {}".format(self.testrun_id))
        top.append(comment)
        return top

    def _properties_element(self, parent_element):
        """Returns properties XML element."""
        testsuites_properties = etree.SubElement(parent_element, "properties")

        etree.SubElement(
            testsuites_properties,
            "property",
            {"name": "polarion-testrun-id", "value": str(self.testrun_id)},
        )
        etree.SubElement(
            testsuites_properties,
            "property",
            {"name": "polarion-project-id", "value": str(self.config["polarion-project-id"])},
        )

        for name, value in sorted(self.config["xunit_import_properties"].items()):
            if name == "polarion-lookup-method":
                lookup_prop = str(value).lower()
                if lookup_prop not in ("id", "name", "custom"):
                    raise Dump2PolarionException(
                        "Invalid value '{}' for the 'polarion-lookup-method' property".format(
                            str(value)
                        )
                    )
                self._lookup_prop = lookup_prop
            elif name in ("polarion-testrun-id", "polarion-project-id"):
                # this was already set
                continue
            else:
                etree.SubElement(
                    testsuites_properties, "property", {"name": name, "value": str(value)}
                )

        return testsuites_properties

    def _fill_lookup_prop(self, testsuites_properties):
        """Fills the polarion-lookup-method property."""
        if not self._lookup_prop:
            raise Dump2PolarionException("Failed to set the 'polarion-lookup-method' property")

        etree.SubElement(
            testsuites_properties,
            "property",
            {"name": "polarion-lookup-method", "value": self._lookup_prop},
        )

    def _testsuite_element(self, parent_element):
        """Returns testsuite XML element."""
        testsuite = etree.SubElement(
            parent_element,
            "testsuite",
            {
                "name": "Import for {} - {} testrun".format(
                    self.config["polarion-project-id"], self.testrun_id
                )
            },
        )
        return testsuite

    @staticmethod
    def _fill_verdict(verdict, result, testcase, records):
        # XUnit Pass maps to Passed in Polarion
        if verdict in Verdicts.PASS:
            records["passed"] += 1
        # XUnit Failure maps to Failed in Polarion
        elif verdict in Verdicts.FAIL:
            records["failures"] += 1
            verdict_data = {"type": "failure"}
            if result.get("comment"):
                verdict_data["message"] = utils.get_unicode_str(result["comment"])
            etree.SubElement(testcase, "failure", utils.sorted_dict(verdict_data))
        # XUnit Error maps to Blocked in Polarion
        elif verdict in Verdicts.SKIP:
            records["skipped"] += 1
            verdict_data = {"type": "error"}
            if result.get("comment"):
                verdict_data["message"] = utils.get_unicode_str(result["comment"])
            etree.SubElement(testcase, "error", utils.sorted_dict(verdict_data))
        # XUnit Skipped maps to Waiting in Polarion
        elif verdict in Verdicts.WAIT:
            records["waiting"] += 1
            verdict_data = {"type": "skipped"}
            if result.get("comment"):
                verdict_data["message"] = utils.get_unicode_str(result["comment"])
            etree.SubElement(testcase, "skipped", utils.sorted_dict(verdict_data))

    def _transform_result(self, result):
        """Calls transform function on result."""
        if self._transform_func:
            result = self._transform_func(result)
        return result or None

    @staticmethod
    def _get_verdict(result):
        """Gets verdict of the testcase."""
        verdict = result.get("verdict")
        if not verdict:
            return None
        verdict = verdict.strip().lower()
        if verdict not in Verdicts.PASS + Verdicts.FAIL + Verdicts.SKIP + Verdicts.WAIT:
            return None
        return verdict

    def _set_lookup_prop(self, result_data):
        """Set lookup property based on processed testcases if not configured."""
        if self._lookup_prop:
            return

        if result_data.get("id"):
            self._lookup_prop = "id"
        elif result_data.get("title"):
            self._lookup_prop = "name"
        else:
            return

        logger.debug("Setting lookup method for xunit to `%s`", self._lookup_prop)

    def _check_lookup_prop(self, result_data):
        """Checks that selected lookup property can be used for this testcase."""
        if not self._lookup_prop:
            return False

        if not result_data.get("id") and self._lookup_prop != "name":
            return False
        if not result_data.get("title") and self._lookup_prop == "name":
            return False
        return True

    @staticmethod
    def _testcase_element(parent_element, result, records, testcase_id, testcase_title):
        """Creates XML element for given testcase result and update testcases records."""
        testcase_time = float(result.get("time") or result.get("duration") or 0)
        records["time"] += testcase_time

        testcase_data = {"name": testcase_title or testcase_id, "time": str(testcase_time)}
        if result.get("classname"):
            testcase_data["classname"] = result["classname"]
        testcase = etree.SubElement(parent_element, "testcase", utils.sorted_dict(testcase_data))
        return testcase

    @staticmethod
    def _fill_out_err(result, testcase):
        """Adds stdout and stderr if present."""
        if result.get("stdout"):
            system_out = etree.SubElement(testcase, "system-out")
            system_out.text = utils.get_unicode_str(result["stdout"])

        if result.get("stderr"):
            system_err = etree.SubElement(testcase, "system-err")
            system_err.text = utils.get_unicode_str(result["stderr"])

    @staticmethod
    def _fill_properties(verdict, result, testcase, testcase_id, testcase_title):
        """Adds properties into testcase element."""
        properties = etree.SubElement(testcase, "properties")
        etree.SubElement(
            properties,
            "property",
            {"name": "polarion-testcase-id", "value": testcase_id or testcase_title},
        )
        if verdict in Verdicts.PASS and result.get("comment"):
            etree.SubElement(
                properties,
                "property",
                {
                    "name": "polarion-testcase-comment",
                    "value": utils.get_unicode_str(result["comment"]),
                },
            )

        params = result.get("params") or {}
        for param, value in params.items():
            etree.SubElement(
                properties,
                "property",
                {
                    "name": "polarion-parameter-{}".format(param),
                    "value": utils.get_unicode_str(value),
                },
            )

    def _gen_testcase(self, parent_element, result, records):
        """Creates record for given testcase result."""
        result = self._transform_result(result)
        if not result:
            return

        verdict = self._get_verdict(result)
        if not verdict:
            logger.warning("Skipping testcase, verdict is missing or invalid")
            return

        testcase_id = result.get("id")
        testcase_title = result.get("title")

        self._set_lookup_prop(result)
        if not self._check_lookup_prop(result):
            logger.warning(
                "Skipping testcase `%s`, data missing for selected lookup method",
                testcase_id or testcase_title,
            )
            return

        testcase = self._testcase_element(
            parent_element, result, records, testcase_id, testcase_title
        )

        self._fill_verdict(verdict, result, testcase, records)
        self._fill_out_err(result, testcase)
        self._fill_properties(verdict, result, testcase, testcase_id, testcase_title)

    def _fill_tests_results(self, testsuite_element):
        """Creates records for all testcases results."""
        if not self.tests_records.results:
            raise NothingToDoException("Nothing to export")

        records = dict(passed=0, skipped=0, failures=0, waiting=0, time=0.0)
        for testcase_result in self.tests_records.results:
            self._gen_testcase(testsuite_element, testcase_result, records)

        tests_num = (
            records["passed"] + records["skipped"] + records["failures"] + records["waiting"]
        )

        if tests_num == 0:
            raise NothingToDoException("Nothing to export")

        testsuite_element.set("errors", str(records["skipped"]))
        testsuite_element.set("failures", str(records["failures"]))
        testsuite_element.set("skipped", str(records["waiting"]))
        testsuite_element.set("time", "{:.4f}".format(records["time"]))
        testsuite_element.set("tests", str(tests_num))

    def export(self):
        """Returns XUnit XML."""
        top = self._top_element()
        properties = self._properties_element(top)
        testsuite = self._testsuite_element(top)
        self._fill_tests_results(testsuite)
        self._fill_lookup_prop(properties)
        return utils.prettify_xml(top)

    def write_xml(self, xml, output_file=None):
        """Outputs the XML content into a file."""
        gen_filename = "testrun_{}-{:%Y%m%d%H%M%S}.xml".format(
            self.testrun_id, datetime.datetime.now()
        )
        utils.write_xml(xml, output_loc=output_file, filename=gen_filename)



"""
Verifies that data were updated in Polarion.
"""

import logging
import os
import time

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


_DEFAULT_TIMEOUT = 600
_DEFAULT_DELAY = 10

_NOT_FINISHED_STATUSES = ("ready", "running")


class QueueSearch:
    """Search for jobs in the completed jobs queue."""

    def __init__(self, session, queue_url, log_url):
        self.session = session
        self.queue_url = queue_url
        self.log_url = log_url
        self.skip = False

    def download_queue(self, job_ids):
        """Downloads data of completed jobs."""
        if self.skip:
            return None

        url = "{}?jobtype=completed&jobIds={}".format(
            self.queue_url, ",".join(str(x) for x in job_ids)
        )
        try:
            response = self.session.get(url, headers={"Accept": "application/json"})
            if response:
                response = response.json()
            else:
                response = None
        # pylint: disable=broad-except
        except Exception as err:
            logger.error(err)
            response = None

        return response

    def find_jobs(self, job_ids):
        """Finds the jobs in the completed job queue."""
        matched_jobs = []
        if self.skip:
            return matched_jobs

        json_data = self.download_queue(job_ids)
        if not json_data:
            return matched_jobs

        jobs = json_data["jobs"]
        for job in jobs:
            if (
                job.get("id") in job_ids
                and job.get("status", "").lower() not in _NOT_FINISHED_STATUSES
            ):
                matched_jobs.append(job)

        return matched_jobs

    def wait_for_jobs(self, job_ids, timeout, delay):
        """Waits until the jobs appears in the completed job queue."""
        found_jobs = []

        if self.skip:
            return found_jobs

        logger.debug("Waiting up to %d sec for completion of the job IDs %s", timeout, job_ids)

        remaining_job_ids = set(job_ids)

        countdown = timeout
        while countdown > 0:
            matched_jobs = self.find_jobs(remaining_job_ids)
            if matched_jobs:
                remaining_job_ids.difference_update({job["id"] for job in matched_jobs})
                found_jobs.extend(matched_jobs)
            if not remaining_job_ids:
                return found_jobs
            time.sleep(delay)
            countdown -= delay

        logger.error(
            "Timed out while waiting for completion of the job IDs %s. Results not updated (yet).",
            list(remaining_job_ids),
        )

    def _check_outcome(self, jobs):
        """Parses returned messages and checks submit outcome."""
        if self.skip:
            return False

        if not jobs:
            logger.error("Import failed!")
            return False

        failed_jobs = []
        for job in jobs:
            status = job.get("status")
            if not status:
                failed_jobs.append(job)
                continue

            if status.lower() != "success":
                failed_jobs.append(job)

        for job in failed_jobs:
            logger.error("job: %s; status: %s", job.get("id"), job.get("status"))
        if len(failed_jobs) == len(jobs):
            logger.error("Import failed!")
        elif failed_jobs:
            logger.error("Some import jobs failed!")
        else:
            logger.info("Results successfully updated!")

        return not failed_jobs

    def _download_log(self, url, output_file):
        """Saves log returned by the message bus."""
        logger.info("Saving log %s to %s", url, output_file)

        def _do_log_download():
            try:
                return self.session.get(url)
            # pylint: disable=broad-except
            except Exception as err:
                logger.error(err)

        # log file may not be ready yet, wait a bit
        for __ in range(5):
            log_data = _do_log_download()
            if log_data or log_data is None:
                break
            time.sleep(2)

        if not (log_data and log_data.content):
            logger.error("Failed to download log file %s.", url)
            return
        with open(os.path.expanduser(output_file), "ab") as out:
            out.write(log_data.content)

    def get_logs(self, jobs, log_file=None):
        """Get log or log url of the jobs."""
        if not (jobs and self.log_url):
            return

        for job in jobs:
            url = "{}?jobId={}".format(self.log_url, job.get("id"))
            if log_file:
                self._download_log("{}&download".format(url), log_file)
            else:
                logger.info("Submit log for job %s: %s", job.get("id"), url)

    def verify_submit(self, job_ids, timeout, delay, **kwargs):
        """Verifies that the results were successfully submitted."""
        if self.skip:
            return False

        jobs = self.wait_for_jobs(job_ids, timeout, delay)
        self.get_logs(jobs, log_file=kwargs.get("log_file"))

        return self._check_outcome(jobs)


def get_queue_obj(session, queue_url, log_url):
    """Checks that all the data that is needed for submit verification is available."""
    skip = False
    if not queue_url:
        logger.error("The queue url is not configured, skipping submit verification")
        skip = True

    if not session:
        logger.error("Missing requests session, skipping submit verification")
        skip = True
    queue = QueueSearch(session=session, queue_url=queue_url, log_url=log_url)
    queue.skip = skip
    return queue


# pylint: disable=too-many-arguments
def verify_submit(
    session, queue_url, log_url, job_ids, timeout=_DEFAULT_TIMEOUT, delay=_DEFAULT_DELAY, **kwargs
):
    """Verifies that the results were successfully submitted."""
    verification_queue = get_queue_obj(session=session, queue_url=queue_url, log_url=log_url)
    return verification_queue.verify_submit(job_ids, timeout, delay, **kwargs)



# -*- coding: utf-8 -*-
"""CSV Unicode Reader."""

import csv


def get_csv_reader(csvfile, dialect=csv.excel, **kwds):
    """Returns csv reader."""
    return csv.reader(csvfile, dialect=dialect, **kwds)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import os

import pytest
from mock import Mock, patch

from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.exporters.xunit_exporter import XunitExport
from dump2polarion.results import ostriztools
from tests import conf


@pytest.fixture(scope="module")
def records_json():
    json_file = os.path.join(conf.DATA_PATH, "ostriz.json")
    return ostriztools.import_ostriz(json_file)


@pytest.fixture(scope="module")
def records_json_cmp():
    json_file = os.path.join(conf.DATA_PATH, "ostriz_cmp.json")
    return ostriztools.import_ostriz(json_file)


@pytest.fixture(scope="module")
def records_json_search():
    json_file = os.path.join(conf.DATA_PATH, "ostriz_search.json")
    return ostriztools.import_ostriz(json_file)


class TestOstriz:
    def test_testrun_id_simple(self):
        testrun_id = ostriztools._get_testrun_id("5.8.0.17")
        assert testrun_id == "5_8_0_17"

    def test_testrun_id_build(self):
        testrun_id = ostriztools._get_testrun_id("5.8.0.17-20170525183055_6317a22")
        assert testrun_id == "5_8_0_17"

    def test_testrun_id_nofill(self):
        testrun_id = ostriztools._get_testrun_id("5.8.0.7-2017")
        assert testrun_id == "5_8_0_7"

    def test_testrun_id_invalid(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools._get_testrun_id("INVALID")
        assert "InvalidVersion parsing testrun ID" in str(excinfo.value)

    def test_duration_good(self):
        duration = ostriztools._calculate_duration(1495766591.151192, 1495768544.573208)
        assert duration == 1953.422016

    def test_duration_bad_finish(self):
        duration = ostriztools._calculate_duration(1495766591.151192, None)
        assert duration == 0

    def test_duration_bad_start(self):
        duration = ostriztools._calculate_duration(0, 1495768544.573208)
        assert duration == 0

    def test_import_orig_data(self, records_json):
        assert hasattr(records_json, "results")
        assert len(records_json.results) == 6
        assert "title" in records_json.results[0]
        assert hasattr(records_json, "testrun")
        assert records_json.testrun == "5_8_0_17"

    def test_invalid_json(self):
        fname = "junit-report.xml"
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools.import_ostriz(os.path.join(conf.DATA_PATH, fname))
        assert "Failed to parse JSON" in str(excinfo.value)

    def test_remote_invalid_json(self):
        with patch("requests.get", return_value=False):
            with pytest.raises(Dump2PolarionException) as excinfo:
                ostriztools.import_ostriz("https://foo")
        assert "Failed to parse JSON" in str(excinfo.value)

    def test_remote_json(self, records_json):
        json_file = os.path.join(conf.DATA_PATH, "ostriz.json")
        with open(json_file, encoding="utf-8") as input_json:
            parsed = input_json.read()
        retval = Mock()
        retval.text = parsed
        with patch("requests.get", return_value=retval):
            loaded_json = ostriztools.import_ostriz("https://foo")
        assert loaded_json == records_json

    def test_no_json(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools.import_ostriz("NONEXISTENT.json")
        assert "Invalid location" in str(excinfo.value)

    def test_empty_json(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools._parse_ostriz("")
        assert "No data to import" in str(excinfo.value)

    def test_e2e_ids_notransform(self, config_prop, records_json):
        exporter = XunitExport(
            "5_8_0_17", records_json, config_prop, transform_func=lambda arg: arg
        )
        complete = exporter.export()
        fname = "ostriz_notransform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_transform(self, config_prop, records_json):
        exporter = XunitExport("5_8_0_17", records_json, config_prop)
        complete = exporter.export()
        fname = "ostriz_transform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_cmp_ids_transform(self, config_prop_cmp, records_json_cmp):
        exporter = XunitExport("5_8_0_17", records_json_cmp, config_prop_cmp)
        complete = exporter.export()
        fname = "ostriz_transform_cmp.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_search_transform(self, config_prop, records_json_search):
        exporter = XunitExport("5_8_0_17", records_json_search, config_prop)
        complete = exporter.export()
        fname = "ostriz_search_transform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed



# pylint: disable=missing-docstring,no-self-use

import os

from dump2polarion.svn_polarion import WorkItemCache
from tests import conf

REPO_DIR = os.path.join(conf.DATA_PATH, "polarion_repo")
WORKITEMS_NUM = 3


class TestWorkItemCache:
    def test_get_all_items(self):
        cache = WorkItemCache(REPO_DIR)
        counter = 0
        for item in cache.get_all_items():
            assert item.get("title")
            assert item.get("type") == "testcase"
            counter += 1
        assert counter == WORKITEMS_NUM



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import copy
import os
from collections import OrderedDict

import pytest

from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporters.requirements_exporter import RequirementExport
from tests import conf

REQ_DATA = [
    OrderedDict(
        (
            ("title", "req01"),
            ("approver-ids", "sbulage:approved"),
            ("assignee", "mkourim"),
            ("category-ids", "CAT-01"),
            ("dueDate", "2018-05-30"),
            ("plannedIn", "PROJ-01"),
            ("initialEstimate", "1/4h"),
            ("priority", "medium"),
            ("severity", "nice_to_have"),
            ("status", "STAT-01"),
            ("reqtype", "functional"),
        )
    ),
    OrderedDict(
        (
            ("title", "req02"),
            ("description", "requirement description"),
            ("assignee-id", "mkourim"),
            ("initial-estimate", "1/4h"),
        )
    ),
    OrderedDict((("id", "PROJ-01"), ("title", "req03"), ("initial-estimate", None))),
    OrderedDict((("id", "PROJ-02"),)),
]


@pytest.fixture(scope="module")
def config_cloudtp(config_prop):
    cloudtp = copy.deepcopy(config_prop)
    cloudtp["polarion-project-id"] = "CLOUDTP"
    cloudtp["requirements-document-relative-path"] = "testing/requirements"
    cloudtp["requirements_import_properties"] = {"prop1": "val1", "prop2": "val2"}
    return cloudtp


class TestRequirement:
    def test_export(self, config_cloudtp):
        req_exp = RequirementExport(REQ_DATA, config_cloudtp)
        complete = req_exp.export()
        fname = "requirement_complete.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_invalid_lookup(self, config_cloudtp):
        new_config = copy.deepcopy(config_cloudtp)
        new_config["requirements_import_properties"] = {"lookup-method": "inv"}
        req_exp = RequirementExport(REQ_DATA, new_config)
        with pytest.raises(Dump2PolarionException) as excinfo:
            req_exp.export()
        assert "Invalid value 'inv' for the 'lookup-method' property" in str(excinfo.value)

    def test_no_requirements(self, config_cloudtp):
        req_exp = RequirementExport([], config_cloudtp)
        with pytest.raises(NothingToDoException) as excinfo:
            req_exp.export()
        assert "Nothing to export" in str(excinfo.value)



# pylint: disable=missing-docstring

from setuptools import find_packages, setup

with open("README.rst", "rb") as fp:
    LONG_DESCRIPTION = fp.read().decode("utf-8").strip()


setup(
    name="dump2polarion",
    use_scm_version=True,
    url="https://github.com/mkoura/dump2polarion",
    description="Dump testcases results to xunit file; submit files to Polarion Importers",
    long_description=LONG_DESCRIPTION,
    author="Martin Kourim",
    author_email="mkourim@redhat.com",
    license="GPL",
    packages=find_packages(exclude=("tests",)),
    entry_points={
        "console_scripts": [
            "csv2sqlite.py = dump2polarion.csv2sqlite_cli:main",
            "polarion_dumper.py = dump2polarion.dumper_cli:main",
        ]
    },
    setup_requires=["setuptools_scm"],
    install_requires=[
        "lxml",
        "pyyaml",
        "requests",
        "docutils",
        "packaging",
        "python-box",
        "polarion-tools-common",
    ],
    keywords=["polarion", "testing"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Intended Audience :: Developers",
    ],
    include_package_data=True,
)



"""
Dump testcases results from a CSV, SQLite, junit, JSON or Ostriz input file to XUnit file.
Submit XUnit, Testcases or Requirements XML to the Polarion Importers.
"""

import argparse
import datetime
import logging
import os

from box import Box

import dump2polarion
from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.results import dbtools

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


def get_args(args=None):
    """Get command line arguments."""
    parser = argparse.ArgumentParser(description="dump2polarion")
    parser.add_argument(
        "-i",
        "--input_file",
        required=True,
        help="Path to CSV, SQLite or JUnit reports file or importers XML file",
    )
    parser.add_argument(
        "-o", "--output_file", help="Where to save the XML output file (default: not saved)"
    )
    parser.add_argument("-t", "--testrun-id", help="Polarion test run id")
    parser.add_argument("-c", "--config-file", help="Path to config YAML")
    parser.add_argument(
        "-n", "--no-submit", action="store_true", help="Don't submit results to Polarion"
    )
    parser.add_argument("--user", help="Username to use to submit results to Polarion")
    parser.add_argument("--password", help="Password to use to submit results to Polarion")
    parser.add_argument("--polarion-url", help="Base Polarion URL")
    parser.add_argument("-f", "--force", action="store_true", help="Don't validate test run id")
    parser.add_argument("--dry-run", action="store_true", help="Dry run, don't update anything")
    parser.add_argument("--no-verify", action="store_true", help="Don't verify import success")
    parser.add_argument(
        "--verify-timeout",
        type=int,
        default=300,
        metavar="SEC",
        help="How long to wait (in seconds) for verification of results submission"
        " (default: %(default)s)",
    )
    parser.add_argument(
        "--job-log", help="Where to save the log file produced by the Importer (default: not saved)"
    )
    parser.add_argument("--log-level", help="Set logging to specified level")
    return parser.parse_args(args)


def get_submit_args(args):
    """Gets arguments for the `submit_and_verify` method."""
    submit_args = dict(
        testrun_id=args.testrun_id,
        user=args.user,
        password=args.password,
        no_verify=args.no_verify,
        verify_timeout=args.verify_timeout,
        log_file=args.job_log,
        dry_run=args.dry_run,
    )
    submit_args = {k: v for k, v in submit_args.items() if v is not None}
    return Box(submit_args, frozen_box=True, default_box=True)


def process_args(args):
    """Processes passed arguments."""
    passed_args = args
    if isinstance(args, argparse.Namespace):
        passed_args = vars(passed_args)
    elif hasattr(args, "to_dict"):
        passed_args = passed_args.to_dict()

    return Box(passed_args, frozen_box=True, default_box=True)


def get_testrun_id(args, config, records_testrun_id):
    """Returns testrun id."""
    config_testrun_id = utils.get_testrun_id_config(config)

    found_testrun_id = args.testrun_id or records_testrun_id or config_testrun_id
    if not found_testrun_id:
        raise Dump2PolarionException(
            "The testrun id was not specified on command line and not found in the input data "
            "or config file."
        )

    match = True
    for tr_id in (args.testrun_id, records_testrun_id, config_testrun_id):
        if tr_id and tr_id != found_testrun_id:
            match = False
            break

    if not match and args.force:
        logger.warning("Using '%s' as testrun id.", found_testrun_id)
    elif not match:
        raise Dump2PolarionException(
            "The test run ids found in exported data, config file and/or specified on command line "
            "differ. If you really want to proceed, add '-f' and test run id '{}' "
            "will be used.".format(found_testrun_id)
        )

    return found_testrun_id


def submit_if_ready(args, submit_args, config):
    """Submits the input XML file if it's already in the expected format."""
    __, ext = os.path.splitext(args.input_file)
    if ext.lower() != ".xml":
        return None

    with open(args.input_file, encoding="utf-8") as input_file:
        xml = input_file.read(1024)

    if not ("<testsuites" in xml or "<testcases" in xml or "<requirements" in xml):
        return None

    # TODO: better detection of xunit file that is ready for import needed
    if "<testsuites" in xml and 'name="pytest"' in xml:
        return None

    if args.no_submit:
        logger.info("Nothing to do")
        return 0

    # expect importer xml and just submit it
    response = dump2polarion.submit_and_verify(
        xml_file=args.input_file, config=config, **submit_args
    )
    return 0 if response else 2


def _get_config(args):
    args_config = {}
    if args.polarion_url:
        args_config["polarion_url"] = args.polarion_url

    return dump2polarion.get_config(args.config_file, args_config)


def dumper(args, config, transform_func=None):
    """Dumper main function."""
    args = process_args(args)
    submit_args = get_submit_args(args)

    submit_outcome = submit_if_ready(args, submit_args, config)
    if submit_outcome is not None:
        # submitted, nothing more to do
        return submit_outcome

    import_time = datetime.datetime.utcnow()

    try:
        records = dump2polarion.import_results(args.input_file, older_than=import_time)
        testrun_id = get_testrun_id(args, config, records.testrun)
        exporter = dump2polarion.XunitExport(
            testrun_id, records, config, transform_func=transform_func
        )
        output = exporter.export()
    except NothingToDoException as info:
        logger.info(info)
        return 0
    except (OSError, Dump2PolarionException) as err:
        logger.fatal(err)
        return 1

    if args.output_file or args.no_submit:
        # when no output file is specified, the 'testrun_TESTRUN_ID-TIMESTAMP'
        # file will be created in current directory
        exporter.write_xml(output, args.output_file)

    if not args.no_submit:
        response = dump2polarion.submit_and_verify(output, config=config, **submit_args)

        __, ext = os.path.splitext(args.input_file)
        if ext.lower() in dbtools.SQLITE_EXT and response:
            dbtools.mark_exported_sqlite(args.input_file, import_time)

        return 0 if response else 2

    return 0


def main(args=None, transform_func=None):
    """Main function for cli."""
    args = get_args(args)

    utils.init_log(args.log_level)

    try:
        config = _get_config(args)
    except Dump2PolarionException as err:
        logger.fatal(err)
        return 1

    return dumper(args, config, transform_func=transform_func)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,comparison-with-callable

import pytest

from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.results.csvtools import import_csv
from dump2polarion.results.dbtools import SQLITE_EXT, import_sqlite
from dump2polarion.results.importer import _get_importer
from dump2polarion.results.junittools import import_junit
from dump2polarion.results.ostriztools import import_ostriz


class TestImporterFormats:
    def test_ostriz_local(self):
        importer = _get_importer("test_ostriz_file.json")
        assert importer == import_ostriz

    def test_ostriz_remote(self):
        importer = _get_importer(
            "http://trackerbot/ostriz/jenkins/jenkins/downstream-58z-extcloud-tests-master"
        )
        assert importer == import_ostriz

    def test_junit(self):
        importer = _get_importer("test_junit.xml")
        assert importer == import_junit

    def test_csv(self):
        importer = _get_importer("workitems.csv")
        assert importer == import_csv

    @pytest.mark.parametrize("ext", SQLITE_EXT)
    def test_db(self, ext):
        importer = _get_importer("workitems{}".format(ext))
        assert importer == import_sqlite

    def test_invalid(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            _get_importer("workitems.txt")
        assert "Cannot recognize type of input data, add file extension" in str(excinfo.value)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import os
from io import StringIO

import pytest

from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.results import csvtools
from tests import conf


class TestCSVFileldNames:
    def test_fieldnames_exported(self):
        csv_file = os.path.join(conf.DATA_PATH, "workitems_ids.csv")
        with open(csv_file, encoding="utf-8") as input_file:
            reader = csvtools._get_csv_reader(input_file)
            fieldnames = csvtools._get_csv_fieldnames(reader)
        assert fieldnames == [
            "id",
            "title",
            "testcaseid",
            "caseimportance",
            "verdict",
            "comment",
            "stdout",
            "stderr",
            "exported",
            "time",
        ]

    def test_fieldnames_unanotated(self):
        csv_content = ",,ID,Title,Test Case I D,Caseimportance"
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        fieldnames = csvtools._get_csv_fieldnames(reader)
        input_file.close()
        assert fieldnames == ["field1", "field2", "id", "title", "testcaseid", "caseimportance"]

    def test_fieldnames_trailing(self):
        csv_content = "ID,Title,Test Case I D,Caseimportance,,,"
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        fieldnames = csvtools._get_csv_fieldnames(reader)
        input_file.close()
        assert fieldnames == ["id", "title", "testcaseid", "caseimportance"]

    def test_fieldnames_missing_id(self):
        csv_content = "Title,Test Case I D,Caseimportance,,,"
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        fieldnames = csvtools._get_csv_fieldnames(reader)
        input_file.close()
        assert fieldnames is None


class TestCSVTestrunId:
    def test_testrun_id_exported(self):
        csv_file = os.path.join(conf.DATA_PATH, "workitems_ids.csv")
        with open(csv_file, encoding="utf-8") as input_file:
            reader = csvtools._get_csv_reader(input_file)
            testrun_id = csvtools._get_testrun_from_csv(input_file, reader)
        assert testrun_id == "5_8_0_17"

    def test_testrun_id_line(self):
        csv_content = str(
            'Query,"(assignee.id:$[user.id] AND NOT status:inactive AND '
            '(TEST_RECORDS:(""RHCF3/5_8_0_17"", @null))) '
            'AND project.id:RHCF3",,,,,,,'
        )
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        testrun_id = csvtools._get_testrun_from_csv(input_file, reader)
        assert testrun_id == "5_8_0_17"

    def test_testrun_id_far(self):
        csv_content = str(
            """
            ,ID,Title,Test Case I D,,,,
            ,(TEST_RECORDS:(""RHCF3/5_8_0_17"", @null)),,,,,"""
        )
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        testrun_id = csvtools._get_testrun_from_csv(input_file, reader)
        assert not testrun_id


class TestCSVImport:
    def test_import_orig_data(self):
        csv_file = os.path.join(conf.DATA_PATH, "workitems_ids.csv")
        data = csvtools.get_imported_data(csv_file)
        assert hasattr(data, "results")
        assert len(data.results) == 15
        assert "id" in data.results[0]
        assert hasattr(data, "testrun")
        assert data.testrun == "5_8_0_17"

    def test_import_no_results(self, tmpdir):
        csv_content = "ID,Title,Test Case I D,Caseimportance,,,"
        csv_file = tmpdir.join("no_results.csv")
        csv_file.write(csv_content)
        csv_file_path = str(csv_file)
        with pytest.raises(Dump2PolarionException) as excinfo:
            csvtools.get_imported_data(csv_file_path)
        assert "No results read from CSV file" in str(excinfo.value)

    def test_import_and_check_verdict(self, tmpdir):
        csv_content = str(
            """
            ,ID,Title,Test Case I D,,,,
            ,RH123,,,,,,"""
        )
        csv_file = tmpdir.join("invalid_fieldnames.csv")
        csv_file.write(csv_content)
        csv_file_path = str(csv_file)
        with pytest.raises(Dump2PolarionException) as excinfo:
            csvtools.import_csv(csv_file_path)
        assert "missing following columns: Verdict" in str(excinfo.value)
